@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Protocol;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.34';
+our $VERSION = '0.35';
 
 use base qw( IO::Async::Notifier );
 
@@ -230,10 +230,9 @@ sub setup_transport
          my $self = shift;
          my ( $transport ) = @_;
 
-         my $on_closed = $self->{on_closed} ||
-                          $self->can( 'on_closed' );
+         $self->maybe_invoke_event( on_closed => );
 
-         $on_closed->( $self ) if $on_closed;
+         $self->configure( transport => undef );
       } ),
    );
 }
